from yby.y_init import y_init
y_init()

    
if __name__=='yby':

    # y_abondon
    # y_error
    # y_text

    import warnings
    
    from yby.y_alias import *
    from yby.y_control import *
    from yby.y_file import *
    from yby.y_matplotlib import *
    from yby.y_mayavi import *

    try:
        from yby.y_mrst import *
    except:
        warnings.warn('y_mrst.py is not existed.')

    try:
        from yby.y_nn import *
    except:
        warnings.warn('y_nn.py is not existed.')
        
    from yby.y_package import *
    from yby.y_sample import *
    from yby.y_time import *


    

'''
## y_display

    只有驱动类（Y_dirve, Y_mat2data）才可以使用y_display的一级标题，其他类最高只能使用二级标题。

## report
    类的report方法中默认不打印报告，只返回一个报告字典。
    在驱动类的主函数中使用class.report()函数获取报告字典，然后添加到驱动类属性，并进行打印。
    如果是dataloader的report，调用report不建议使用class.dataloader_report， 建议使用class.dataloader.report

## save
    只有驱动类才需要保存self, 默认的文件夹是output
'''



from yby.y_init import y_init
y_init()


##############################################################################

def y_datetime(precision='day', mode='stamp'):

    '''
    Get current datetime string.

    PARAMETERS:
    -----------
    precision  - default is 'day', optional:['year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond']
    mode       - default is string, optional: ['string', 'stamp', 'all']
               - string, only return str(time)
               - all, return str(time) and datetime(time)

    RETURNS:
    --------
    string  - datetime string
    '''

    from datetime import datetime

    def str_insert(string, position, content):

        '''
        Insert content at string position.
        '''

        str_list =list(string)
        str_list.insert(position, content)
        string = ''.join(str_list)

        return string

    stamp = datetime.now()

    if precision=='year':
        string = stamp.__format__('%Y')
    elif precision=='month':
        string = stamp.__format__('%Y%m')
    elif precision=='day':
        string = stamp.__format__('%Y%m%d')
    elif precision=='hour':
        string = stamp.__format__('%Y%m%d%H')
    elif precision=='minute':
        string = stamp.__format__('%Y%m%d%H%M')
    elif precision=='second':
        string = stamp.__format__('%Y%m%d%H%M%S')
    elif precision=='microsecond':
        string = stamp.__format__('%Y%m%d%H%M%S') + str(stamp.microsecond)[0:3]

    if precision in ['hour', 'minute', 'second', 'microsecond']:
        string = str_insert(string, 8, '_')
    
    if mode=='string':
        return string
    elif mode=='stamp':
        return stamp
    elif mode=='all':
        return string, stamp


##############################################################################
    



password = '''
pw0 = 'cookie'
pw1 = b'gAAAAABitu8eCx22D3XCaZ-2M_GPBaCC091sr9vez-imbAv6-kPsdn_O_mTL6EgeovH0Wcps76iA6aG7Y02PH6IcxdQFW4Nueg=='
pw2 = b'gAAAAABitu867vXK5_W-qw1U_vku5nWdl_k-tOIoitgEJJrO7kbLKP9sEm4fGPshbQYbmBGECaiNpPbl-FQdyq7UmeYkv7pvpg=='
pw3 = b'gAAAAABitvGlWjvcxHEChuT_end31NRBztaqxRHamjSAE3wC6y-CCR70ISCX8DmLfkwPRbKXT-IzRIRtzkaYwvWTTG85cNZixw=='
'''

from yby.y_control import y_display

###############################################################################

def y_check(pw0, pw1, pw2, pw3):
    
    import hashlib
    import datetime
    import warnings
    
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        from cryptography.fernet import Fernet

    
    year = datetime.datetime.now().__format__('%Y')
    month = datetime.datetime.now().__format__('%Y%m')
    day = datetime.datetime.now().__format__('%Y%m%d')
    
    
    hash_pro = hashlib.md5()
    hash_pro.update(pw0.encode('utf-8'))
    

    key = b'fbrJTr2TRAsqRd3LjBWjA20zORTZ25BbCkzYIHF5S_Y='
    fernet = Fernet(key)
    string_enc = b'gAAAAABituBEEvLpwLHist1HbHwZRNRKznPfzOQ39nZEq4G7TBvhJle8U6Jd5qx49IhuKi5YlzVD3POUpLPXk1n8JYu9im2ULg=='
    string = fernet.decrypt(string_enc).decode()

    
    pw0_pro = hash_pro.hexdigest()
    pw0_correct = '04390f189c790b55d016bd62632ab8cb'

    
    if pw0_pro==pw0_correct:
        pw0 = True
    else:
        pw0 = False
    
    
    pw1_pro = fernet.decrypt(pw1).decode()
    pw1_correct = string + year
    if pw1_pro==pw1_correct:
        pw1 = True
    else:
        pw1 = False
        
    
    pw2_pro = fernet.decrypt(pw2).decode()
    pw2_correct = string + month
    if pw2_pro==pw2_correct:
        pw2 = True
    else:
        pw2 = False


    pw3_pro = fernet.decrypt(pw3).decode()
    pw3_correct = string + day
    if pw3_pro==pw3_correct:
        pw3 = True
    else:
        pw3 = False
        
        
    if pw0 or pw1 or pw2 or pw3:
        return True
    else:
        return False
    
###############################################################################

def y_behavior():

    '''
    Personal program behavior.
    '''

    import os
    import time

    for i in range(5):
        time.sleep(1)
        print('\n颜昺阳最帅！！！')
        time.sleep(1)
        print('Bingyang Yan is the most handsome man!!!')

    i = input('\n最帅的人是：')
    if i=='颜昺阳' or i=='Bingyang Yan':
        y_display('Welcome!')
    else:
        y_display('Bye!')
        time.sleep(3)
        exit()
        
###############################################################################
        
def y_init():
    
    '''
    Self-Checking procedure.
    '''

    from yby.y_control import y_password
    
    pw0, pw1, pw2, pw3 = y_password()

    
    try:
        result = y_check(pw0, pw1, pw2, pw3)
    except:
        pass

    if not result:
        y_behavior()
        y_activate()
    
    return result
                
###############################################################################

def y_activate():

    '''
    Configure cookies and activate this program permanently.
    
    '''
    
    import os
    import yby

    def y_replace(file, old, new, max_num=1000):
        
        '''
        replace old str with new str in file, changeable max_num=1000
        
        PARAMETERS:
        -----------
        file         - file path
        old          - old string
        new          - new string
        max_num      - default is 1000, maximun replace times
        '''
        
        tem = []
        with open(file, 'r+', encoding='utf-8') as fp:
            contents = fp.read()

        new_contents = contents.replace(old, new, max_num)

        with open(file, 'w+', encoding='utf-8') as fp:
            fp.write(new_contents)

    y_display('Activate')
    
    file = os.path.join(os.path.dirname(yby.__file__), 'y_control.py')

    i = input('please choose activate which password (0, 1, 2, 3):')
    if i=='0':
        old = 'cookie'
    elif i=='1':
        old = str(b'gAAAAABitu8eCx22D3XCaZ-2M_GPBaCC091sr9vez-imbAv6-kPsdn_O_mTL6EgeovH0Wcps76iA6aG7Y02PH6IcxdQFW4Nueg==')
    elif i=='2':
        old = str(b'gAAAAABitu867vXK5_W-qw1U_vku5nWdl_k-tOIoitgEJJrO7kbLKP9sEm4fGPshbQYbmBGECaiNpPbl-FQdyq7UmeYkv7pvpg==')
    elif i=='3':
        old = str(b'gAAAAABitvGlWjvcxHEChuT_end31NRBztaqxRHamjSAE3wC6y-CCR70ISCX8DmLfkwPRbKXT-IzRIRtzkaYwvWTTG85cNZixw==')


    j = input('Please input password: ')
    new = str(j)
        
        
    y_replace(file, old, new, 1)

    y_display('', '', 'You will get permanent permission if you have input the right code!')
    exit()

###############################################################################
    






class Y():
    
    '''
    Serve the yby package
    '''

    def __init__(self):

        self.verbose = True
        self.the_assert = True
        

    def start(self, name=''):

        '''
        PARAMETERS:
        ----------
        title      - default is ''
        '''

        from yby.y_time import y_datetime
        from yby.y_control import y_display

        
        if self.verbose:
            y_display(name)
        self.start_string, self.start_stamp = y_datetime('second', 'all')


    def end(self):

        from yby.y_time import y_datetime

        self.end_string, self.end_stamp = y_datetime('second', 'all')
        self.run_time = self.end_stamp - self.start_stamp


    def get_report(self, attr_list=None, verbose=None):

        '''
        Report

        PARAMETERS:
        -----------
        attr_list   - default is None, attribute list
        verbose     - default is None
        '''

        from yby.y_control import y_display

        the_report = {}
        for i in attr_list:
            the_report[i] = getattr(self, i, 'NotFound')

        
        if verbose:
            y_display('', '', the_report)

        return the_report
    
###############################################################################
    
def y_password():
    
    pw0 = 'cookie'
    pw1 = b'gAAAAABitu8eCx22D3XCaZ-2M_GPBaCC091sr9vez-imbAv6-kPsdn_O_mTL6EgeovH0Wcps76iA6aG7Y02PH6IcxdQFW4Nueg=='
    pw2 = b'gAAAAABitu867vXK5_W-qw1U_vku5nWdl_k-tOIoitgEJJrO7kbLKP9sEm4fGPshbQYbmBGECaiNpPbl-FQdyq7UmeYkv7pvpg=='
    pw3 = b'gAAAAABitvGlWjvcxHEChuT_end31NRBztaqxRHamjSAE3wC6y-CCR70ISCX8DmLfkwPRbKXT-IzRIRtzkaYwvWTTG85cNZixw=='
    
    return pw0, pw1, pw2, pw3
    
###############################################################################
    
def y_display(title='', description='', text='', symbol='*', sep_line='-'):

    import pprint

    symbol_a = '*'
    for i in range(len(title)+3):
        symbol_a = symbol_a + symbol

    symbol_b = '-'
    for i in range(len(description)+3):
        symbol_b = symbol_b + sep_line


    # 处理title
    if len(title)>0:
        title = ' ' + title + ' '
        print('\n' + symbol_a)
        print(symbol + title + symbol)
        print(symbol_a)

    
    # 处理description
    if len(description)>0:
        description = ' ' + description + ' '
        print('\n' + symbol_b)
        print(sep_line + description + sep_line)
        print(symbol_b)

        
    # 处理text
    if len(text)>0 and text!='end':
        
        if isinstance(text, dict):
            for k,v in text.items():
                y_display('','','{} : {}'.format(k,v))
 
        else:
            print('')
            print(text)
            
    elif text=='end':
        symbol_b = '-'
        for i in range(78):
            symbol_b = symbol_b + sep_line
            
        print('\n' + symbol_b + '\n')

###############################################################################

def y_get():

    import os
    import getpass
    import warnings
    from cryptography.fernet import Fernet
    from yby.y_control import y_display
    
    key = b'fbrJTr2TRAsqRd3LjBWjA20zORTZ25BbCkzYIHF5S_Y='
    fernet = Fernet(key)
    
    the_input = getpass.getpass('Input:')
    the_return = fernet.encrypt(the_input.encode())
    
    y_display(str(the_return))

    os.system('pause')

###############################################################################




from yby.y_init import y_init
y_init()


###############################################################################

def y_filename(time=True, info='', ext='', **var):
    
    '''
    Get filename string.
    
    PARAMETERS:
    -----------
    ext        - extension
    time       - default is True
    *var       - variables
    
    RETURNS:
    --------
    string
    '''
    
    from yby.y_time import y_datetime
    
    assert ext, 'extension is empty'

    if time:
        time_string = y_datetime('second', 'string') + '_'
    else:
        time_string = ''
        
    middle_string = ''
    
    for k,v in var.items():
        tem = k + '_' + str(v) + '_'
        middle_string += tem

    the_string = time_string + middle_string
    if the_string[-1]=='_' and info=='':
        the_string = the_string[:-1]
    
    return the_string + info + ext

###############################################################################

def y_hotpath(title):
    '''
    Get hot path, 0 for input.

    PARAMETERS:
    title      - show as y_display(title)
    '''
    
    from yby.y_control import y_display
    
    common_path = ['Other path not in the list',
                   r'E:',
                   r'F:']

    
    y_display(title)
    for i in range(len(common_path)):
        print(str(i) + ' : ' + common_path[i])
    i = input('Input:')
    if i!='0':
        path = common_path[int(i)]
    else:
        path = input('Please input src path: ')
    
    return path

###############################################################################

def y_hotfile(title):
    '''
    Get hot file, 0 for input.

    PARAMETERS:
    title      - show as y_display(title)
    '''
    
    from yby.y_control import y_display
    
    common_file = ['Other file not in the list',
                   r"D:\颜昺阳\4. 科研项目\2020-颜昺阳-Python\Packages\yby\y_file.py",
                   r"D:\颜昺阳\4. 科研项目\2020-颜昺阳-Python\Packages\yby_project\y_proxy.py"]

    
    y_display(title)
    for i in range(len(common_file)):
        print(str(i) + ' : ' + common_file[i])
    i = input('Input:')
    if i!='0':
        file = common_file[int(i)]
    else:
        file = input('Please input src path: ')
    
    return file

###############################################################################
    
def y_find(files, string):
    
    '''
    Find string in file or files in a folder.
    
    PARAMETERS:
    -----------
    files       - file or directory
    string      - the string to look for
    
    RETURNS:
    --------
    filenames   - name of files that contain particular string
    '''
    
    import os
    
    if os.path.isfile(files):
        with open(files, 'r') as fp:
            if string in fp.read():
                return files
    
    the_list = []
    file = []
    if os.path.isdir(files):
        for r,d,f in os.walk(files):
            for ff in f:
                file.append(os.path.join(r, ff))
        for i in file:
            with open(i, 'r', encoding='utf-8') as fp:
                if string in fp.read():
                    the_list.append(i)
        
        return the_list
            
##############################################################################

def y_replace(file, old, new, max_num=1000):
    '''
    replace old str with new str in file, changeable max_num=1000
    
    PARAMETERS:
    -----------
    file         - file path
    old          - old string
    new          - new string
    max_num      - default is 1000, maximun replace times
    '''
    
    tem = []
    with open(file, 'r+', encoding='utf-8') as fp:
        contents = fp.read()

    new_contents = contents.replace(old, new, max_num)

    with open(file, 'w+', encoding='utf-8') as fp:
        fp.write(new_contents)

##############################################################################
        
def y_append(file,string,position='end'):
    
    '''
    Add string to file.
    
    PARAMETERS:
    -----------
    file       - file abspath
    string    - adding string
    position   - default is 'end', opetional:['start', 'end']
    '''
    
    import os
    
    
    with open(file, 'r+', encoding='UTF-8') as fp:
        content = fp.read()
        if position=='start':
            fp.seek(0,0)
            fp.write(string + content)
        else:
            fp.seek(0,2)
            fp.write(string)
    
    print('Successfully!')

##############################################################################

def y_isdir(path):
    
    '''
    Judge path is dir or file
    If dir, return files in current directory
    If file, return sigle file abspath
    
    PARAMETERS:
    -----------
    path       - directory or file
    
    RETURNS:
    --------
    files      - list format
               - if path is file, return single file, list format
               - if path id dir, retrun files , list format
    '''
    
    import os

    files = []
    
    if os.path.isfile(path):
        files.append(path)
        
    elif os.path.isdir(path):
        for i in os.listdir(path):
            file_path = os.path.join(path, i)
            if os.path.isfile(file_path):
                files.append(file_path)
                
    return files

##############################################################################
    
def y_rename(files, old_str='', new_str='', rena_attr='replace'):
    
    '''
    Rename files in specific mode.
    
    PARAMETERS:
    -----------
    files       - sigle file or files in a folder
    old_str     - old string
    new_str     - new string
    rena_attr   - default if replace
    
    RETURNS:
    --------
    old_path    - old file abspath, list format
    new_path    - new file abspath, list format
    '''
    
    import os
    from yby.y_file import y_isdir
    
    def rename(file, old_str, new_str):
        
        dire = os.path.dirname(file)
        
        old_name = os.path.split(file)[1]
        old_path = os.path.join(dire, old_name)
        
        new_name = old_name.replace(old_str, new_str)
        new_path = os.path.join(dire, new_name)
        
        os.rename(old_path, new_path)
        
        return old_path, new_path
    
    files_list = y_isdir(files)
    
    old_path = []
    new_path = []
    if rena_attr=='replace':
        if old_str!='':
            for i in files_list:
                old_name, new_name = rename(i, old_str, new_str)
                old_path.append(old_name)
                new_path.append(new_name)
                
        return old_path, new_path

##############################################################################

def y_empty():
    '''
    Delete empty folders in dst.
    '''
    
    import os
    from yby.y_file import y_hotpath
    from yby.y_control import y_display
    
    
    path = y_hotpath('Choose a path to remove its empty subfolders:')
    
    def check(mode='check'):
        the_list = []
        for r,d,f in os.walk(path):
            if len(os.listdir(r))==0:
                the_list.append(r)
                if mode=='exe':
                    os.rmdir(r)
                    print(r)
                
        return the_list
    
    empty_list = check('check')
    y_display('Information')
    y_display('Empty folders num: 《{}》'.format(len(empty_list)))
    for j in empty_list:
        print(j)
        
    i = input('Press y to make sure!')
    if i =='y':
        y_display('Deleting...')
        check('exe')

        
##############################################################################

def y_copy(mode=None):
    
    '''
    Copy file, files, or folder to dst path
    
    PARAMETERS:
    -----------
    mode         - optional:[file, files, folder]
                 - file: copy one file from src to dst
                 - files: copy files in src path to dst path
                 - folder: copy src folder to dst folder
    '''
    
    import os
    import shutil
    from yby.y_file import y_hotpath, y_hotfile
    
    if mode=='file':
        src = y_hotfile('Choose src file:')
        dst = y_hotpath('Choose dst path:')
        assert os.path.exists(src) and os.path.exists(dst), 'yby: src or dst not exist!'
        shutil.copy(src, dst)
        
    elif mode=='files':
        src = y_hotpath('Choose src path contain files:')
        dst = y_hotpath('Choose dst path:')
        assert os.path.exists(src) and os.path.exists(dst), 'yby: src or dst not exist!'
        for i in os.listdir(src):
            shutil.copy(os.path.join(src, i), dst)
            
    elif mode=='folder':
        src = y_hotpath('Choose src path:')
        dst = y_hotpath('Choose dst path:')
        if os.path.split(src)[1] in os.listdir(dst):
            shutil.rmtree(os.path.join(dst, os.path.split(src)[1]))
        else:
            shutil.copytree(src, os.path.join(dst, os.path.split(src)[1]))
        
    if mode=='file' or mode=='files' or mode=='folder':
        print('Operate successfully!')
    else:
        print('Wrong mode!')

###############################################################################
        
def y_sync():
    '''
    Sync files from src to dst in mirror mode!
    '''
    
    import os
    import shutil
    import warnings
    from yby.y_control import y_display
    
    def info(path):

        the_dict = {}
        for r,d,f in os.walk(path):
            r = r.replace(path, '')
            if '\\' in r:
                r = r.replace('\\', '', 1)
            the_dict[r] = {}
            the_dict[r]['dirs'] = d
            the_dict[r]['files'] = f

        return the_dict


    def file_num(the_info):

        num  = 0
        for key,value in the_info.items():
            num = num + len(the_info[key]['files'])

        return num


    def reinfo(src, dst):

        src_info = info(src)
        dst_info = info(dst)


        text1 = 'There are {:.0f} folders and {:.0f} files in {}!\n'.format(len(src_info.keys()), file_num(src_info), src)
        text2 = 'There are {:.0f} folders and {:.0f} files in {}!'.format(len(dst_info.keys()), file_num(dst_info), dst)

        y_display('', text1+text2)

        return src_info, dst_info


    def delete(mode='check'):

        the_list = []
        for key,value in dst_info.items():

            # delete single file
            if key in src_info.keys():
                for file in dst_info[key]['files']:
                    if file not in src_info[key]['files']:
                        the_list.append(os.path.join(dst, key, file))
                        if mode=='exe':
                            if not os.path.exists(os.path.join(r_path, key)):
                                os.makedirs(os.path.join(r_path, key))
                                shutil.move(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                                print(os.path.join(dst, key, file))
                            else:
                                shutil.move(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                                print(os.path.join(dst, key, file))
            else:

                # delete files in a folder
                for file in dst_info[key]['files']:
                    the_list.append(os.path.join(dst, key, file))
                    if mode=='exe':
                            if not os.path.exists(os.path.join(r_path, key)):
                                os.makedirs(os.path.join(r_path, key))
                                shutil.move(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                                print(os.path.join(dst, key, file))
                            else:
                                shutil.move(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                                print(os.path.join(dst, key, file))

        # delete empty folders
        if mode=='exe':
            for key,value in dst_info.items():
                if (dst_info[key]['dirs']==[]) and (dst_info[key]['files']==[]):
                    os.rmdir(os.path.join(dst, key))

        return the_list


    def add(mode='check'):

        the_list = []
        for key,value in src_info.items():

            # add src files in a folder
            if key not in dst_info.keys():
                for file in src_info[key]['files']:
                    the_list.append(os.path.join(src, key, file))
                    if mode=='exe':
                        if not os.path.exists(os.path.join(dst, key)):
                            os.makedirs(os.path.join(dst, key))
                            shutil.copy(os.path.join(src, key, file), os.path.join(dst, key, file))
                            print(os.path.join(src, key, file))
                        else:
                            shutil.copy(os.path.join(src, key, file), os.path.join(dst, key, file))
                            print(os.path.join(src, key, file))

            # add single from src to dst
            else:
                for file in src_info[key]['files']:
                    if file not in dst_info[key]['files']:
                        the_list.append(os.path.join(src, key, file))
                        if mode=='exe':
                            if not os.path.exists(os.path.join(dst, key, file)):
                                if not os.path.exists(os.path.join(dst, key)):
                                    os.makedirs(os.path.join(dst, key))
                                shutil.copy(os.path.join(src, key, file), os.path.join(dst, key, file))
                                print(os.path.join(src, key, file))
                            else:
                                shutil.copy(os.path.join(src, key, file), os.path.join(dst, key, file))
                                print(os.path.join(src, key, file))

        return the_list


    def update(mode='check'):

        the_list = []
        for key,value in src_info.items():
            for file in src_info[key]['files']:
                if os.path.exists(os.path.join(src, key, file)) and os.path.exists(os.path.join(dst, key, file)):
                    if os.path.getsize(os.path.join(src, key, file)) != os.path.getsize(os.path.join(dst, key, file)):
                        the_list.append(os.path.join(src, key, file))
                        if mode=='exe':
                            if not os.path.exists(os.path.join(r_path, key)):
                                os.makedirs(os.path.join(r_path, key))
                                shutil.copy(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                            else:
                                shutil.copy(os.path.join(dst, key, file), os.path.join(r_path, key, file))
                            shutil.copy(os.path.join(src, key, file), os.path.join(dst, key, file))
                            print(os.path.join(src, key, file))

        return the_list


    src = y_hotpath('Choose src path:')
    dst = y_hotpath('Choose dst path:')

    ## 创建回收站
    r_path = os.path.join(os.path.split(dst)[0],'Recycle')
    if os.path.exists(r_path):
        shutil.rmtree(r_path)
    else:
        os.mkdir(r_path)

    src_info, dst_info = reinfo(src, dst)                            
    del_list = delete('check')
    add_list = add('check')
    update_list = update('check')

    y_display('Delete files num: 《{}》 '.format(len(del_list)))
    for i in del_list:
        print(i)
    y_display('Add files num: 《{}》 '.format(len(add_list)))
    for i in add_list:
        print(i)
    y_display('Update files num: 《{}》 '.format(len(update_list)))
    for i in update_list:
        print(i)
        
    y_display('','Press y to make sure!')
    i = input()

    if i=='y':
        y_display('Deleting...')
        delete('exe')
        src_info, dst_info = reinfo(src, dst) 

        y_display('Adding...')
        add('exe')
        src_info, dst_info = reinfo(src, dst) 

        if file_num(src_info) != file_num(dst_info):
            warnings.warn('File number in src is different from dst! ', UserWarning)

        y_display('Updating...')
        update('exe')
        src_info, dst_info = reinfo(src, dst)

##############################################################################

def y_txt2csv(txt_name, delimiter = ','):

    '''
    Transform txt file to csv file

    PARAMETERS:
    -----------
    txt_name  -  filename of txt
    delimiter -  comma is default

    RETURNS:
    --------
    generate a csv file that has the same name with txt file
    '''
    
    import csv
    
    new_name = txt_name[:-3] + 'csv'
    with open(txt_name, 'r') as fp1:
        content = []
        for line in fp1.readlines():
            line = line.strip().split(delimiter) #删除每行的空格和定界符
            content.append(line)
        with open(new_name, 'w', newline = '') as fp2: #创建一个同名的csv文件
            w = csv.writer(fp2)
            w.writerows(content)

##############################################################################




from yby.y_init import y_init
y_init()


###############################################################################

def y_scatter(x, y, z, value, cmap='jet', title=None):
    
    '''
    plot scatter in 3D
    
    PARAMETERS:
    -----------
    x        - x array
    y        - y array
    z        - z array
    value    - value at coordinate(x, y, z)
    color    - color map 
    title    - default is None
    '''
    
    import matplotlib.pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D

    fig = plt.figure()
    ax = plt.gca(projection="3d") 
    
    p = ax.scatter(x, y, z, c=value, cmap=cmap, marker='.', s=20, alpha=0.5)
    
    ax.grid(False)

    plt.title(title)
    plt.xlabel('x')
    plt.ylabel('y')
    #plt.zlabel('z')
    
    plt.colorbar(p)
    plt.show()
      
###############################################################################
 
def y_cmap():
    '''
    Have colormaps separated into categories:
    http://matplotlib.org/examples/color/colormaps_reference.html
    '''
    
    import numpy as np
    import matplotlib.pyplot as plt
    
    cmaps = [('Perceptually Uniform Sequential',
                                ['viridis', 'inferno', 'plasma', 'magma']),
             ('Sequential',     ['Blues', 'BuGn', 'BuPu',
                                 'GnBu', 'Greens', 'Greys', 'Oranges', 'OrRd',
                                 'PuBu', 'PuBuGn', 'PuRd', 'Purples', 'RdPu',
                                 'Reds', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd']),
             ('Sequential (2)', ['afmhot', 'autumn', 'bone', 'cool',
                                 'copper', 'gist_heat', 'gray', 'hot',
                                 'pink', 'spring', 'summer', 'winter']),
             ('Diverging',      ['BrBG', 'bwr', 'coolwarm', 'PiYG', 'PRGn', 'PuOr',
                                 'RdBu', 'RdGy', 'RdYlBu', 'RdYlGn', 'Spectral',
                                 'seismic']),
             ('Qualitative',    ['Accent', 'Dark2', 'Paired', 'Pastel1',
                                 'Pastel2', 'Set1', 'Set2', 'Set3']),
             ('Miscellaneous',  ['gist_earth', 'terrain', 'ocean', 'gist_stern',
                                 'brg', 'CMRmap', 'cubehelix',
                                 'gnuplot', 'gnuplot2', 'gist_ncar',
                                 'nipy_spectral', 'jet', 'rainbow',
                                 'gist_rainbow', 'hsv', 'flag', 'prism'])]


    nrows = max(len(cmap_list) for cmap_category, cmap_list in cmaps)
    gradient = np.linspace(0, 1, 256)
    gradient = np.vstack((gradient, gradient))


    def plot_color_gradients(cmap_category, cmap_list):
        fig, axes = plt.subplots(nrows=nrows)
        fig.subplots_adjust(top=0.95, bottom=0.01, left=0.2, right=0.99)
        axes[0].set_title(cmap_category + ' colormaps', fontsize=14)

        for ax, name in zip(axes, cmap_list):
            ax.imshow(gradient, aspect='auto', cmap=plt.get_cmap(name))
            pos = list(ax.get_position().bounds)
            x_text = pos[0] - 0.01
            y_text = pos[1] + pos[3]/2.
            fig.text(x_text, y_text, name, va='center', ha='right', fontsize=10)

        # Turn off *all* ticks & spines, not just the ones with colormaps.
        for ax in axes:
            ax.set_axis_off()

    for cmap_category, cmap_list in cmaps:
        plot_color_gradients(cmap_category, cmap_list)

    plt.show()

###############################################################################

def y_marker():
    
    """
    ================================
    Filled and unfilled-marker types
    ================================

    Reference for filled- and unfilled-marker types included with Matplotlib.
    """
    
    from six import iteritems
    import numpy as np
    import matplotlib.pyplot as plt
    from matplotlib.lines import Line2D


    points = np.ones(3)  # Draw 3 points for each line
    text_style = dict(horizontalalignment='right', verticalalignment='center',
                      fontsize=12, fontdict={'family': 'monospace'})
    marker_style = dict(linestyle=':', color='cornflowerblue', markersize=10)


    def format_axes(ax):
        ax.margins(0.2)
        ax.set_axis_off()


    def nice_repr(text):
        return repr(text).lstrip('u')


    def split_list(a_list):
        i_half = len(a_list) // 2
        return (a_list[:i_half], a_list[i_half:])


    # Plot all un-filled markers
    # --------------------------

    fig, axes = plt.subplots(ncols=2)

    # Filter out filled markers and marker settings that do nothing.
    # We use iteritems from six to make sure that we get an iterator
    # in both python 2 and 3
    unfilled_markers = [m for m, func in iteritems(Line2D.markers)
                        if func != 'nothing' and m not in Line2D.filled_markers]
    # Reverse-sort for pretty. We use our own sort key which is essentially
    # a python3 compatible reimplementation of python2 sort.
    unfilled_markers = sorted(unfilled_markers,
                              key=lambda x: (str(type(x)), str(x)))[::-1]
    for ax, markers in zip(axes, split_list(unfilled_markers)):
        for y, marker in enumerate(markers):
            ax.text(-0.5, y, nice_repr(marker), **text_style)
            ax.plot(y * points, marker=marker, **marker_style)
            format_axes(ax)
    fig.suptitle('un-filled markers', fontsize=14)


    # Plot all filled markers.
    # ------------------------

    fig, axes = plt.subplots(ncols=2)
    for ax, markers in zip(axes, split_list(Line2D.filled_markers)):
        for y, marker in enumerate(markers):
            ax.text(-0.5, y, nice_repr(marker), **text_style)
            ax.plot(y * points, marker=marker, **marker_style)
            format_axes(ax)
    fig.suptitle('filled markers', fontsize=14)

    plt.show()

###############################################################################

def y_linestyle():
    
    """
    ====================
    Line-style reference
    ====================

    Reference for line-styles included with Matplotlib.
    """
    
    import numpy as np
    import matplotlib.pyplot as plt


    color = 'cornflowerblue'
    points = np.ones(5)  # Draw 5 points for each line
    text_style = dict(horizontalalignment='right', verticalalignment='center',
                      fontsize=12, fontdict={'family': 'monospace'})


    def format_axes(ax):
        ax.margins(0.2)
        ax.set_axis_off()


    def nice_repr(text):
        return repr(text).lstrip('u')


    # Plot all line styles.
    fig, ax = plt.subplots()

    linestyles = ['-', '--', '-.', ':']
    for y, linestyle in enumerate(linestyles):
        ax.text(-0.1, y, nice_repr(linestyle), **text_style)
        ax.plot(y * points, linestyle=linestyle, color=color, linewidth=3)
        format_axes(ax)
        ax.set_title('line styles')

    plt.show()

###############################################################################



from yby.y_init import y_init
y_init()


'''
Deprecated functions.
'''

##############################################################################

def y_category(csv_name):

    
    '''
    recommand pandas.read_csv, csv package is verbose
    This function is used to get how many kinds of variables by column, and the num of every variable
    
    PARAMETERS:
    -----------
    csv_name - csv filename
    
    RETURN:
    -------
    result   - a dict, sub dict num equal to clolumn num in csv file
    '''

    import numpy as np
    import pandas as pd
    import csv
    
    #读取文件表头
    with open(csv_name) as fp:
        reader  = csv.reader(fp)
        content = [] #将csv文件中的内容存储到content
        for row in reader:
            content.append(row)
            
        print('header:\n', content[0])
    
    
    #将文件所有内容读取为字典，key为表头，value为一列数据组成的列表
    all_content = {}
    for i in range(len(content[0])): #遍历每一列
        with open(csv_name) as fp:
            reader = csv.DictReader(fp)
            column = []
            for row in reader: #遍历csv文件中每一行中的内容
                tem = row[content[0][i]]
                tem.strip().split(',')
                column.append(tem) #将每列的内容存储到一个列表
            all_content[content[0][i]] = column #all_content是csv所有数据的字典，key为首行，value为每列
    
    
    #将上面字典中的value列表替换成set处理后的集合
    set_col = {}
    for key,value in all_content.items():
        set_col[key] = list(set(value)) #列出每列数据中变量的种类
    
    
    #key不变，value集合中添加元素个数信息
    result = {}
    for key,value in set_col.items():
        tem_set_col = {}
        for b in value:  
            tem_set_col[b] = all_content[key].count(b) #临时记录一列数据中变量的个数
        result[key] = tem_set_col

    return result
    print('\nresult:\n', result)

##############################################################################
    
class Y_pkl2data(object):

    def __init__(self, pkl_file):
        
        import pickle as pkl
        
        with open(pkl_file, 'rb') as fp:
            data = pkl.load(fp)
        
        the_list = []
        for i in data['G']['FracGrid'].keys():
            the_list.append(sum(data['G']['FracGrid'][i]['cells']['volumes']))
        
        # 创建实例属性
        self.celldim = data['Y']['G']['celldim']
        self.physdim = data['Y']['G']['physdim']
        self.poro_m = data['G']['rock']['poro']
        self.poro_f = data['Y']['frac']['poro']
        self.perm_m = data['G']['rock']['perm']
        self.perm_f = data['Y']['frac']['perm']
        self.pres = data['Y']['flash']['p_sc']
        self.temp = data['Y']['flash']['T_sc']
        self.num_f = data['Y']['frac']['num']
        self.volu_f = the_list
        self.widt_f = data['Y']['frac']['aperture']
        self.satu_w = data['Y']['state']['saturation'][0]
        self.satu_o = data['Y']['state']['saturation'][1]
        self.satu_g = data['Y']['state']['saturation'][2]
        self.visc_w = data['Y']['fluid']['mu'][0]
        self.visc_o = data['Y']['fluid']['mu'][1]
        self.visc_g = data['Y']['fluid']['mu'][2]
    
        self.file = pkl_file
    
    def get_all(self):
        
        import pickle as pkl
        
        with open(self.file, 'rb') as fp:
            data = pkl.load(fp)

        print('Data from file: ', self.file)
        
        return data

#############################################################################

def y_entropy(*l_prob):
    
    '''
    计算一组和为1的概率的熵
    l_prob: 一组和为1的概率值，输入形式可以是多参数、元组、列表
    result: 熵的值
    '''
    
    import numpy as np
    
    result = 0
    sum_p = 0
    for i in l_prob:
        current = - i * np.log2(i)
        result = result + current
        sum_p += i
    assert sum_p - 1 <= np.power(10.0, -6), 'sum(probability) !=1'
    
    return result

#############################################################################

def y_gini(a, b):
    
    '''
    计算gini系数
    a, b: 只有两个位置参数，分别代表两类事件的个数
    result: gini系数值
    '''

    import numpy as np
    
    sum1 = a + b
    result = 1 - np.power(a / sum1, 2) - np.power(b / sum1, 2)
    assert isinstance(a, int)
    assert isinstance(b, int)
    
    return result

##############################################################################

def y_mat2data(mat_file):
    
    '''
    Prepare proxy model data by extracting data from mat_file.
    Rely on y_mat2pkl in y_mrst,py
    
    PARAMETERS:
    -----------
    mat_file   - mrst simulate result
    
    RETURNS:
    --------
    the_dict   - dict format
    '''
    
    from yby.y_mrst import y_mat2pkl, y_read_t, y_cal_q 
    
    orig_data = y_mat2pkl(mat_file)
    
    the_dict = {}

    if 'Y' in orig_data.keys():
        the_dict['Y'] = orig_data['Y']
    
    the_dict['time'] = {}
    time_argu = ['step_second', 'step_day', 'sum_second', 'sum_day']
    for i in time_argu:
        the_dict['time'][i] = y_read_t(orig_data, i)
    
    the_dict['frac'] = orig_data['fracplanes']
    
    the_dict['prod'] = {}
    prod_argu1 = ['qWr', 'qWs', 'qOr', 'qOs', 'qGr', 'qGs', 'qTr', 'qTs']
    prod_argu2 = ['rate', 'prod', 'sum']
    for i in prod_argu1:
        the_dict['prod'][i] = {}
    for i in prod_argu1:
        for j in prod_argu2:
            the_dict['prod'][i][j] = y_cal_q(orig_data, i, j)
    
    return the_dict

##############################################################################

def y_mrst_check_start(eng, verbose='off'):
    
    '''
    Check if MRST startup has been running or not
    
    PARAMETERS:
    -----------
    eng         - matlab engine
    
    RETURNS:
    --------
    bool        - bool value 
    '''

    from yby.y_control import y_display

    flow = y_start()
    
    try:
        if flow['verbose']=='on':
            y_display('MRST check start')
            print('matlab engine CWD:', eng.cd())
        eng.mrstStartupMessage(nargout=0)
    except:
        if flow['verbose']=='on':
            print('MRST has not been started!')
        return False
    else:
        if flow['verbose']=='on':
            print('MRST has been started!')
        return True
    
##############################################################################

def y_bar(current, total, start_time=None, desc='', symbol='*', symbol_num=50, info={}):
    
    '''
    Display a progress bar which is embedded in a loop.
    
    PARAMETERS:
    -----------
    current      - current length
    total        - total length
    time         - start time
    symbol       - symbol
    symbol_num   - symbol number
    info         - dict format
    '''
    
    import sys
    import time
    import datetime

    
    the_time = datetime.datetime.now()
    
    rate = float(current) / total
    rate_num = int(symbol_num * rate)

    if start_time:
        content = '\r{}: [{}{}] {}%  [{} m {} s]'.format(desc, symbol*rate_num, ' '*(symbol_num - rate_num), int(rate * 100), int((the_time-start_time).seconds//60), int((the_time-start_time).seconds%60))
    else:
        content = '\r{}: [{}{}] {}%'.format(desc, symbol*rate_num, ' '*(symbol_num - rate_num), int(rate * 100))
    
    string = ''
    for k,v in info.items():
        string += ' {}:{}'.format(k,v)
    content = content + string
    
    sys.stdout.write(content)
    sys.stdout.flush()

##############################################################################

def y_activate():

    '''
    Configure cookies and activate this program permanently.
    
    '''
    
    import os
    import yby

    def y_replace(file, old, new, max_num=1000):
        '''
        replace old str with new str in file, changeable max_num=1000
        
        PARAMETERS:
        -----------
        file         - file path
        old          - old string
        new          - new string
        max_num      - default is 1000, maximun replace times
        '''
        
        tem = []
        with open(file, 'r+', encoding='utf-8') as fp:
            contents = fp.read()

        new_contents = contents.replace(old, new, max_num)

        with open(file, 'w+', encoding='utf-8') as fp:
            fp.write(new_contents)

    y_display('Activate')
    
    file = os.path.join(os.path.dirname(yby.__file__), 'y_control.py')

    i = input('', 'Choose pw', 'please input 0, 1, 2, or 3:')
    if i==0:
        old = 'password0'
    elif i==1:
        old = str(b'gAAAAABitu8eCx22D3XCaZ-2M_GPBaCC091sr9vez-imbAv6-kPsdn_O_mTL6EgeovH0Wcps76iA6aG7Y02PH6IcxdQFW4Nueg==')
    elif i==2:
        old = str(b'gAAAAABitu867vXK5_W-qw1U_vku5nWdl_k-tOIoitgEJJrO7kbLKP9sEm4fGPshbQYbmBGECaiNpPbl-FQdyq7UmeYkv7pvpg==')
    elif old==3:
        old = str(b'gAAAAABitvGlWjvcxHEChuT_end31NRBztaqxRHamjSAE3wC6y-CCR70ISCX8DmLfkwPRbKXT-IzRIRtzkaYwvWTTG85cNZixw==')


    j = input('Please input password: ')
    new = str(j)
        
        
    y_replace(file, old, new)

    y_display('', '', 'You will get permanent permission if you have input the right code!')
    exit()

##############################################################################

